% Function which illustrates May's bifircations in the logistic equation
% x(n+1) = rx(n)*( 1 - x(n) ). In this version, using the arrow keys
% will increment or decrement r, with limits of 0 < r < 4.
function may_with_keypress
global d
d.r = 3.5;    %Growth parameter
d.dr = 0.01;  %Increment in growth parameter
d.N = 50;     %Number of iterations
d.M = 300;    %Number of x starting values between 0 and 1
fsize = 18; %Fontsize

%

%Determine default x iterations
[d.iter,d.x] = may( d.r,d.N,d.M );

%Initial plot
figure('name','May bifurcations','color',[1 1 1],...
    'units','normalized','position',[0 0 1 1],'KeyPressFcn',@keypress)
d.plot = plot( d.iter, d.x,'.' ); ylim([0,1]);
xlabel('Iteration','fontsize',d.fsize); ylabel('x','fontsize',fsize)
set(gca,'fontsize',fsize); grid on;
d.title = title(['r = ',num2str(d.r)],'fontsize',fsize);

%Set colours to match colormap, based upon initial x values
for n=1:length( d.plot )
    [R,G,B] = colfromcolormap( d.x(1,n),0,1 );
    set( d.plot(n),'color', [R,G,B] );
end

%Output a PNG image
print( gcf, ['May r=',strrep(num2str(d.r),'.','p'),'.png'],'-r300','-dpng' );

%%

%Use May's logistic function to determine N iterations (rows) of the M
%columns of x values, with different initial conditions within [0,1]
function [iter,x] = may(r,N,M)

%Create vector of initial values of x
xx = linspace(0.01,0.99,M);

%Create a vector of iteration numbers, and replicate over M columns.
iter = 0 : 1 : N; iter = repmat( iter.', [1,M] );

%Use May's iterative formula to compute subsequent x values
x = xx;
for n=1:N
    xx = r*xx.*(1-xx); 
    x = [x;xx];
end

%%

%Plot update function
function keypress_update
global d
[d.iter,d.x] = may( d.r,d.N,d.M );
for n=1:numel(d.plot)
    set( d.plot(n),'xdata',d.iter(:,n),'ydata',d.x(:,n) );
end
set(d.title,'string',['r = ',num2str(d.r)]);

%%

%Function which executes when a key is pressed
function keypress(fig,evnt)
global d
if strcmp(get(fig,'currentkey'),'p')==1
    %Print a 300DPI PNG image
    print(gcf,['May  r=',strrep(num2str(d.r),'.','p'),'.png'],...
        '-dpng','-r300');
elseif strcmp(get(fig,'currentkey'),'uparrow')==1
    %Increment r
    d.r = d.r + d.dr; if d.r >4; d.r = 4; end
    keypress_update
elseif strcmp(get(fig,'currentkey'),'downarrow')==1
    %Decrement r
    d.r = d.r - d.dr; if d.r <0 ; d.r = 0; end
    keypress_update
end

%%

%Get R,G,B colour interpolated from current colormap
function [R,G,B] = colfromcolormap(x,xmin,xmax)

%Get colormap and extract vectors of red, green, blue intensities
cmap = colormap; r = cmap(:,1).'; g = cmap(:,2).'; b = cmap(:,3).';

%Interpolate colormap, using x within range [xmin,xmax]
xx = linspace( xmin, xmax, length(r) );
if x > xmax; x = xmax; end;  if x < xmin; x = xmin; end;
R = interp1( xx, r, x ); G = interp1( xx, g, x ); B = interp1( xx, b, x );

%End of code

